#!/bin/sh

[ -n "$include_widgets" ] && return 0
include_widgets="$0"

. "$_EXEC/cgilite/storage.sh"

w_refuri="$(URL "$PATH_INFO")?$(HTML "$QUERY_STRING")"

w_ascii='
  s-&#48;-0-g; s-&#49;-1-g; s-&#50;-2-g; s-&#51;-3-g; s-&#52;-4-g; s-&#53;-5-g;
  s-&#54;-6-g; s-&#55;-7-g; s-&#56;-8-g; s-&#57;-9-g;
  s-&#65;-A-g; s-&#66;-B-g; s-&#67;-C-g; s-&#68;-D-g; s-&#69;-E-g; s-&#70;-F-g;
  s-&#71;-G-g; s-&#72;-H-g; s-&#73;-I-g; s-&#74;-J-g; s-&#75;-K-g; s-&#76;-L-g;
  s-&#77;-M-g; s-&#78;-N-g; s-&#79;-O-g; s-&#80;-P-g; s-&#81;-Q-g; s-&#82;-R-g;
  s-&#83;-S-g; s-&#84;-T-g; s-&#85;-U-g; s-&#86;-V-g; s-&#87;-W-g; s-&#88;-X-g;
  s-&#89;-Y-g; s-&#90;-Z-g;
  s-&#97;-a-g; s-&#98;-b-g; s-&#99;-c-g; s-&#100;-d-g; s-&#101;-e-g; s-&#102;-f-g;
  s-&#103;-g-g; s-&#104;-h-g; s-&#105;-i-g; s-&#106;-j-g; s-&#107;-k-g; s-&#108;-l-g;
  s-&#109;-m-g; s-&#110;-n-g; s-&#111;-o-g; s-&#112;-p-g; s-&#113;-q-g; s-&#114;-r-g;
  s-&#115;-s-g; s-&#116;-t-g; s-&#117;-u-g; s-&#118;-v-g; s-&#119;-w-g; s-&#120;-x-g;
  s-&#121;-y-g; s-&#122;-z-g;
  s-&#45;-\--g; s-&#47;-/-g; s-&#47;-/-g; s-&#58;-:-g; s-&#61;-=-g; s-&#64;-@-g;
  s-&#95;-_-g; s-&#126;-~-g;              s-&#32;- -g; s-&#94;-^-g; s-&#124;-|-g;
'

w_tags="$_DATA/.index/tags.cache"; w_tagcategories="$_DATA/.index/tagcategories.cache"
if [ ! -f "$w_tags" -o ! -f "$w_tagcategories" ] \
   || [ "$(find "$_DATA/" -path '*/.index/meta' -newer "$w_tags")" ]
  then
  w_tags="$( { local cn=1
    find "$_DATA/" -path '*/.index/meta' -print0 \
    | xargs -r0 sed -r '
      s;^.*\t.*\t.*\ttags=(.*)\tcomment=.*\t.*\r$;\1;;
      s;,;\n;g;'"$UNSTRING" \
    | { sort; printf '\n'; } \
    | while read -r tag; do
      [ "$tag" = "$otag" ] \
      && cn=$((cn + 1)) \
      || {
        printf "%i	%s\n" "$cn" "$otag"
        cn=1
      }
      otag="$tag"
    done
  } |sort -rn |cut -f2- |HTML |sed "$w_ascii s-&#10;-\n-g; s;\n\n;\n;g;" |tee "$w_tags" )"
  w_tagcategories="$(printf %s "$w_tags" \
                     | cut -sd: -f1 |sort -u \
                     | tee "$w_tagcategories" )"
else
  w_tags="$(cat "$w_tags")"
  w_tagcategories="$(cat "$w_tagcategories")"
fi


[ "$ORDER" = Name   ] && w_coname=checked
[ "$ORDER" = Date   ] && w_codate=checked
[ "$ORDER" = Length ] && w_colength=checked

w_bookmarks(){
  local name='' cf='' cs='' bm="$_DATA/.index/bookmarks" proposed_name=''
  [ ! -d "${bm%/*}" ] && return 0
  [ ! -f "$bm" ] && touch "$bm"

  [ "$SEARCH" ] && cs="$(STRING "$SEARCH")"
  [ "$FILTER" ] && cf="$(STRING "$FILTER")"

  name="$(grep -m1 -aF "	search=$cs	filter=$cf${CR}" "$bm" |cut -f1 |UNSTRING)"
  [ ! "$name" ] && proposed_name="$(
    printf '%s\t%s' "$SEARCH" "$FILTER" \
    |sed -r '/^\t$/{  s;\t;All;; q;}
             /.*\t$/{ s;\t$;;; q;}
             /^\t.*/{ s;^\t;;;
                      :x; s;(^|[~^|])([^|^~:]+):;\1;; tx;
                      s;\^; and ;g; s;\|;,;g; s;~;not ;g; q;}'
    )"

  printf '[form #bookmarks action=?a=bookmark method=POST
            [a href="#" x]
            [hidden "ref" "%s"]
            [hidden "search" "%s"][hidden "filter" "%s"]
            [label Name for current page:]
            [input name="name" value="%s" placeholder="Name" ]
            [button type="submit" %s]' \
            "$w_refuri" \
            "$(HTML "$SEARCH")" "$(HTML "$FILTER")" \
            "$(HTML "${name:-${proposed_name}}")" \
            "${name:+Update}${proposed_name:+Add}"
  [ "$name" ] && printf ' [submit "delete" "delete" Delete]'

  sort "$bm" |while read -r name search filter; do
    search="${search#search=}" filter="${filter#filter=}" filter="${filter%${CR}}"
    [ "$search" = "${cs}" -a "$filter" = "${cf}" ] && continue

    name="$(UNSTRING "$name")";
    search="$(UNSTRING "${search}" |URL)";
    filter="$(UNSTRING "${filter}" |URL)";
    printf '[label .link %s]
            [a .link target=blank href="?o=Name&s=%s&f=%s" by Name]
            [a .link target=blank href="?o=Date&s=%s&f=%s" by Date]
            [a .link target=blank href="?o=Length&s=%s&f=%s" by Length]
            [br]' \
            "$(HTML "$name" |sed 's;&#44\;;&[wbr];g;')" \
            "$search" "$filter" \
            "$search" "$filter" \
            "$search" "$filter"
  done
  printf ']'
}

w_search(){
  printf '
  [form #search method=GET action=?
    [select name=o size=1 
      [option disabled=disabled Order By]
      [option value=Name %s Name]
      [option value=Date %s Date]
      [option value=Length %s Length]
    ]
    [input name=s placeholder=Search value="%s"]
  ]
  ' \
  "$w_coname" "$w_codate" "$w_colength" \
  "$(HTML "$SEARCH")"
}

w_prefs(){
  local tm tf td

  tm=''; [ "$(COOKIE mode)" = index ] && tm=' '
  tf=''; [ "$(COOKIE fakemp4)" = yes ] && tf=checked
  td=''; [ "$(COOKIE downscale)" = yes ] && td=checked

  printf '
  [form #prefs method="POST" action="?a=setprefs"
    [a href="#" x]
    [hidden "ref" "%s"]
    [label for=prefs_ps Pagesize]
    [input #prefs_ps type=number name=pagesize value="%s"][br]
    [radio "mode" "browse" %s #prefs_modebrowse] [label for=prefs_modebrowse Browse Folders][br]
    [radio "mode" "index"  %s #prefs_modeindex ] [label for=prefs_modeindex View Full Index][br]
    [checkbox "fakemp4" "yes" %s #prefs_fmp4] [label for=prefs_fmp4 Fake .MP4 file type][br]
    [checkbox "downscale" "yes" %s #prefs_downscale] [label for=prefs_downscale Prefer downscale to 480p][br]
    [submit "index" "update" Force Index Update][br]
    [submit "store" "store" Set Cookie]
  ]
  ' \
  "$w_refuri" "$LISTSIZE" \
  "${tm:-checked}" "${tm:+checked}" "$tf" "$td"
}

w_index(){
  printf '
  [form #index method="POST" action="?a=spawnindex"
    [hidden "ref" "%s"]
    [label Set up for Index view: ]
    [checkbox "recursive" "yes" #spawn_recursive] [label for=spawn_recursive Include subdirectories]
    [submit "spawn" "spawn" Set up]
  ]
  ' "$w_refuri"
  return 0
}

w_advsearch(){
  local n lbid tag category filter f t
  filter="$(HTML "${FILTER}^" |sed "$w_ascii")"

  printf '[form #advsearch action=?a=advsearch method=POST
            [a href="#" Hide]
            [p .help Select multiple tags from each category by holding down the [strong Ctrl] key on your keyboard.[br]
            Refine the search further by setting additional search tags using the [strong "+and"] button.]'

  for n in 1 2 3 4 5 6 7 8 9 10; do
    f="${filter%%^*}"; filter="${filter#*^}"

    t=''; [ "$f" -a ! "${f%%~*}" ] && t=" "

    lbid="cat_${n}_(none)"
    printf '[input .and type=checkbox name=and id="and_%i" %s][label for="and_%i" +and
            ][fieldset .select
            [radio "pol_%i" "pos" .pol %s #pol_pos_%i"][label for=pol_pos_%i Any]
            [radio "pol_%i" "neg" .pol %s #pol_neg_%i"][label for=pol_neg_%i None]
            [label .head Category:]' \
            $n "${f:+checked}" $n \
            $n "${t:-checked}" $n $n \
            $n "${t:+checked}" $n $n

    f="|${f#~}|"
    printf '*\n%s\n' "$w_tagcategories" \
    | while read -r category; do
      lbid="cat_${n}_${category}"

      t=''
      [ "$category"  = '*' -a   "${f%%|${category}:*}" ] && t=checked
      [ "$category" != '*' -a ! "${f%%|${category}:*}" ] && t=checked

      printf '[radio "cat_%i" "%s" .cat %s id="%s"][label for="%s" %s]
              [select name=tag_%s size=10 multiple' \
              $n "$category" "$t" "$lbid" "$lbid" "$category" $n

      printf '%s\n' "$w_tags" \
      | { [ "$category" = '*' ] && grep -avF ':' || grep -awF "${category}"; } \
      | { for n in 1 2 3 4 5 6 7 8 9 0; do
          read -r line && printf '%s\n' "$line" || break;
          done;  # path 10 lines through without modification
          sort;  # and sort remaining lines
      } | while read -r tag; do
        [ "$tag" ] || continue
        t=''; [ ! "${f%%*|${tag}|*}" ] && t=checked
        printf '[option %s value="%s"\n%s]' "$t" "$tag" "${tag#*:}"
      done
      printf '\n]'
    done
    printf ']'
  done

  printf '[fieldset .submit [select name=order
            [option disabled=disabled Order By]
            [option value=Name %s Name]
            [option value=Date %s Date]
            [option value=Length %s Length]
          ][button type=submit Apply Filter]]
          ]' \
          "$w_coname" \
          "$w_codate" \
          "$w_colength"
}

w_tagging(){
  local tag category
  printf '[a href="#multitag" Add Tags / Remove Tags]
          [div #multitag [input type="hidden" name="ref" value="%s"]' "$w_refuri"
  printf '[a href="#" Hide][br]'

  printf 'Tags\n%s\n' "$w_tagcategories" \
  | while read -r category; do
    [ "$category" ] || continue
    printf '[fieldset [legend %s:][select name=tag size=4 multiple\n' "$category"
    printf %s "$w_tags" \
    | { [ "$category" = 'Tags' ] && grep -avF ':' || grep -awF "${category}"; } \
    | { for n in 1 2 3 4 5 6 7 8 9 0; do
        read -r line && printf '%s\n' "$line" || break
        done;
        sort;
    } | while read -r tag; do
      [ "$tag" ] || continue
      printf '[option value="%s"\n%s]' "$tag" "${tag#*:}"
    done
    printf ']]'
  done

  printf '[fieldset [legend New:][textarea name=newtag\n]
          [submit "op" "del" Remove Tags][submit "op" "add" Add Tags]
          ]]'
}
